%% script to print a COMSOL-compatible input file with the emission coefficients for the analysed emission lines. It is created taking as base
%% an ASCII file, TAB delimited generated by a spectral line database (e.g. NIST) with the emission parameters characterizing each line
%% The data needs to be arranged in five columns, the first for wavelength in nm (labeled wl), the second for the transition probability in 1/s (labeled gA),
%% the third for the lower energy level in eV (labeled El), the fourth for the upper energy level in eV (labelered Eu) and the fifth column for the FWHM of the line in nm (labeled FWHM)
PathFile=pwd;                      %folder of the files with the source data
%% User input data
Filename='AtomicSpectraDataLinesNIST_forCuII_420_780nm_Edited_TAB.dat';                   %Name of the files with the source data
OutputCOMSOLVariableFileName='AtomicSpectralDataCOMSOL_CuII.txt';        %Name of the COMSOL variable file that will be generated here
SpecieID='CuII';                %Name of the emitting specie
% Filename='AtomicSpectraDataLinesNIST_forCuI_420_650nm_Edited_TAB.dat';
% OutputCOMSOLVariableFileName='AtomicSpectralDataCOMSOL_CuI.txt';        %Name of the COMSOL variable file that will be generated here
% SpecieID='CuI';                %Name of the emitting specie

%% start of the routine
DataEmitter = tdfread([PathFile,'\',Filename]); 
nLines=size(DataEmitter.wl);
fileID = fopen(OutputCOMSOLVariableFileName,'w');

%physical constants used
q_E=1.60217657e-19;     %Elemental charge in C
h_Plank=6.62606957E-34; %Plank's constant in m2 kg/s
c=299792458;        %Speed of light in m/s
EpsilonEquation='';
try
    disp(['Now compiling spectral line data into a file... please wait']);
    for j=1:nLines
        LambdaC_identifier=['LambdaC_',SpecieID,'_',num2str(j)];
        formatSpec = [LambdaC_identifier,' %s "Central wavelength (m) peak ',num2str(j),'" \r\n'];
        fprintf(fileID,formatSpec,DataEmitter.wl(j)*1e-9);  %notice that we multiply by 1e-9 to convert nm to m
        FWHM_identifier=['FWHM_',SpecieID,'_',num2str(j)]; formatSpec = [FWHM_identifier,' %s "Full with half maximum FWHM (m) for peak ',num2str(j),'" \r\n'];
        fprintf(fileID,formatSpec,DataEmitter.FWHM(j)*1e-9);  %notice that we multiply by 1e-9 to convert nm to m
        Eu_identifier=['Eu_',SpecieID,'_',num2str(j)]; formatSpec = [Eu_identifier,' %s "Energy (J) upper level for peak ',num2str(j),'" \r\n'];
        fprintf(fileID,formatSpec,DataEmitter.Eu(j)*q_E);    %notice that we multiply for qe to convert ev to J
        hcgA_4piwl_identifier=['hcgA_4piwl_',SpecieID,'_',num2str(j)];
        formatSpec = [hcgA_4piwl_identifier,' %s "Constant term h*c*gA/(4*pi*lambdaC) for peak ',num2str(j),'" \r\n'];
        hcgA_4piwl=h_Plank*c*DataEmitter.gA(j)/(4*pi*DataEmitter.wl(j)*1e-9);
        fprintf(fileID,formatSpec,hcgA_4piwl);  
        LProfile_identifier=['L_profile_',SpecieID,'_',num2str(j)];
        formatSpec = [LProfile_identifier,' (2/pi)*(',FWHM_identifier,'/(4*(Lambda_y-',LambdaC_identifier,')^2+',FWHM_identifier,'^2)) "Lorentz profile for peak ',num2str(j),'" \r\n'];
        fprintf(fileID,formatSpec);  
        Epsilon_identifier=['Epsilon_',SpecieID,'_',num2str(j)];
        formatSpec = [Epsilon_identifier, ' ',hcgA_4piwl_identifier,'*exp(-',Eu_identifier,'/(k_B*T))*(','n_s','/','Z_s',')*',LProfile_identifier,' "Emission coefficient for peak ',num2str(j),'" \r\n'];
        fprintf(fileID,formatSpec);  
        EpsilonEquation=[EpsilonEquation,'+',Epsilon_identifier];
    end
    formatSpec = ['Epsilon_sum', ' ',EpsilonEquation, ' "Summation of emission coefficient for all peaks \r\n'];
    fprintf(fileID,formatSpec);  
    fclose(fileID);
    disp(['Spectral data has succesfully compiled in the file: ',OutputCOMSOLVariableFileName]);
catch
    disp(['Spectral data has filed to compile. Last error detected: ',lasterr,' . Check your input file']);
end
